<?php

abstract class am4_Settings_Abstract implements ArrayAccess, Iterator
{
    protected $settings= [];
    protected $position;

    public function __construct()
    {
        $this->load();
        $this->rewind();
    }

    public function get($key, $default = null)
    {
        if($this->isEmpty()) return $default;
        if(array_key_exists($key, $this->settings)) return  $this->settings[$key];
        return $default;
    }

    function __get($name)
    {
        return $this->get($name);
    }

    function __set($key, $value)
    {
        return $this->set($key, $value);
    }

    public function set($key, $value){
        if (is_null($key)) {
            $this->settigns[] = $value;
        } else {
            $this->settings[$key] = $value;
        }
        return $this;
    }

    abstract public function save();
    abstract public function load();

    function isEmpty()
    {
        return empty($this->settings);
    }

    function loadFromArray(Array $array){
        $this->settings = $array;
        return $this;
    }

    function loadDefaults(Array $array)
    {
        foreach($array as $k =>$v){
            $this->set($k, $v);
        }
        return $this;
    }

    function delete($key)
    {
        if(array_key_exists($key, $this->settings)) unset($this->settings[$key]);
        return $this;
    }

    #[\ReturnTypeWillChange]
    public function offsetSet($offset, $value)
    {
        if (is_null($offset)) {
            $this->settigns[] = $value;
        } else {
            $this->settings[$offset] = $value;
        }
    }

    #[\ReturnTypeWillChange]
    public function offsetExists($offset)
    {
        return isset($this->settings[$offset]);
    }

    #[\ReturnTypeWillChange]
    public function offsetUnset($offset)
    {
        unset($this->settigns[$offset]);
    }

    #[\ReturnTypeWillChange]
    public function offsetGet($offset)
    {
        $val = (isset($this->settings[$offset]) ? $this->settings[$offset] : null);
        return $val;
    }

    #[\ReturnTypeWillChange]
    function rewind()
    {
        if(is_array($this->settings))
        {
            reset($this->settings);
            $this->position = key($this->settings);
        }
    }

    #[\ReturnTypeWillChange]
    function current()
    {
        return current($this->settings);
    }

    #[\ReturnTypeWillChange]
    function key()
    {
        return key($this->settings);
    }

    #[\ReturnTypeWillChange]
    function next()
    {
        next($this->settings);
        $this->position = key($this->settings);
    }

    #[\ReturnTypeWillChange]
    function valid()
    {
        return isset($this->settings[$this->position]);
    }

}

class am4_Settings_Config extends am4_Settings_Abstract
{
    /**
     * Name of option in database;
     */
    protected $name = 'am4options';

    function __construct($name=null)
    {
        if(!empty($name)) $this->name=$name;
        parent::__construct();
    }

    public function save()
    {
        update_option($this->name, $this->settings);
    }

    public function load()
    {
        $this->settings = get_option($this->name, []);
        return $this;
    }
}

class am4_Settings_Post_Meta extends am4_Settings_Abstract
{
    protected $name= 'am4options';
    protected $post_id;

    public function __construct($post_id=null, $key='am4options')
    {
        $this->name = $key;
        $this->setPostId($post_id);
        parent::__construct();
    }

    public function load()
    {
        if ($this->post_id){
            $this->settings = get_post_meta($this->post_id, $this->name, true) ?: [];
            $_ = get_post_meta($this->post_id);
            ksort($_);
            foreach($_ as $k => $v)
            {
                if(strpos($k, $this->name."_") !== false)
                {
                    [,$key]= explode($this->name."_", $k);
                    $this->settings = array_merge($this->settings, [$key => $v[0]]);
                }
            }
        }
        return $this;
    }

    public function save()
    {
        if(empty($this->post_id))
            throw new Exception('Post ID is empty. Nothing to save!');

        $_ = $this->settings;
        foreach($_ as $k=>$v)
        {
            if(!is_array($v))
            {
                update_post_meta($this->post_id, $this->name."_".$k, $v);
            }
        }
        update_post_meta($this->post_id, $this->name, $_);
    }

    function setPostId($post_id)
    {
        $this->post_id = $post_id;
        return $this;
    }

    function getPostId()
    {
        return $this->post_id;
    }
}
class am4_Settings_Post_Meta_Array extends am4_Settings_Post_Meta
{
    protected $name= 'am4options';
    protected $post_id;

    public function load()
    {
    }

    public function save()
    {
    }
    static function create($post_id, $settings)
    {
        $_ = new self($post_id);

        // To fix wordpress bug with double serialization;
        $settings = maybe_unserialize($settings);

        // It seems data is corrupted still.
        $_->loadFromArray(is_array($settings)?$settings:[]);

        return $_;
    }

}


class am4_Settings_Category extends am4_Settings_Abstract
{
    /**
     * Name of option in database;
     */
    protected $name = 'am4catoptions';
    protected $category;
    /**
     * @var am4_Settings_Config
     */
    protected $config;

    function __construct($category)
    {
        $this->category = $category;
        $this->config = new am4_Settings_Config($this->name);
        parent::__construct();
    }

    public function save()
    {
        $this->config->set($this->category, $this->settings);
        $this->config->save();
    }

    public function load()
    {
        $this->settings = $this->config->get($this->category, []);
        return $this;
    }
}

class am4_Settings_Tag extends am4_Settings_Abstract
{
    /**
     * Name of option in database;
     */
    protected $name = 'am4tags';
    protected $tag;
    /**
     * @var am4_Settings_Config
     */
    protected $config;

    function __construct($tag)
    {
        $this->tag = $tag;
        $this->config = new am4_Settings_Config($this->name);
        parent::__construct();
    }

    public function save()
    {
        $this->config->set($this->tag, $this->settings);
        $this->config->save();
    }

    public function load()
    {
        $this->settings = $this->config->get($this->tag, []);
        return $this;
    }
}

class am4_Settings_Error extends am4_Settings_Config
{
    /**
     * Name of option in database;
     */
    protected $name = 'am4errors';

    function getTextByName($name)
    {
        foreach($this->settings as $k=>$v){
            if(is_array($v) && array_key_exists("name", $v) && $v['name'] == $name) return @$v['text'];
        }
        return null;
    }

    function add(Array $value)
    {
        $this->settings[] = $value;
        return $this;
    }
}

class am4_Settings_Templates extends am4_Settings_Config
{
    /**
     * Name of option in database;
     */
    protected $name = 'am4styles';
}