<?php


class am4UpgradeDb
{
    protected $hooks;
    
    function add(string $version, callable $callback) : self
    {
        $this->hooks[$version][] = $callback;
        return $this;
    }
    
    function wpdb() : wpdb
    {
        return $GLOBALS['wpdb'];
    }
    function getDbVersion() : ?string
    {
        return am4PluginsManager::getOption('db_version')??"0.0";
    }
    
    function setDbVersion($version) : void
    {
        am4PluginsManager::getAllOptions()->set('db_version', $version)->save();
    }
    
    function getPluginVersion() : string
    {
        if( ! function_exists('get_plugin_data') )
        {
            require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        }
        
        $version = get_plugin_data(AM4_PLUGIN_DIR."/amember4.php");
        
        return $version['Version']??"";
        
    }
    function __invoke()
    {
        if(is_admin())
        {
            if(version_compare($this->getPluginVersion(), $this->getDbVersion(), '>'))
            {
                ksort($this->hooks);
                foreach($this->hooks as $version => $list)
                {
                    if(version_compare($version, $this->getDbVersion(), '<=')){
                        continue;
                    }
                    foreach($list as $callback)
                    {
                        if(is_callable($callback)){
                            call_user_func($callback, $this->getDbVersion());
                        }
                    }
                }
                $this->setDbVersion($this->getPluginVersion());
                
            }
            
        }
    }
}


$upgradeDb = new am4UpgradeDb();

$upgradeDb->add('1.8.2', function($version) use($upgradeDb){
    $results = $upgradeDb->wpdb()->get_col("select distinct(post_id) from {$upgradeDb->wpdb()->postmeta} where meta_key = 'am4options'");
    foreach($results as $post_id){
        (new am4_Settings_Post_Meta($post_id))->load()->save();
    }
});


add_action("plugins_loaded", $upgradeDb);
